#pragma once

#include <cstdint>
#include <vector>
#include <filesystem>

namespace ReArchive {
    class FileEntry;
}

class ReArchive::FileEntry {
protected:
    //uint64 string length.
    std::filesystem::path path;
    int64_t data_size;
    int64_t data_offset;
public:
    [[nodiscard]] std::filesystem::path Path() const { return path; }
    [[nodiscard]] int64_t Size() const { return data_size; }
    [[nodiscard]] int64_t Offset() const { return data_offset; }
public:
    /// @param data_size The number of bytes long the file is.
    /// @param data_offset How many bytes away from the beginning of the archive this file is stored.
    /// @param file_path The file name including any path you want to be included in the archive.
    FileEntry(const int64_t& data_size, const int64_t& data_offset, std::filesystem::path path) : path(std::move(path)), data_size(data_size), data_offset(data_offset)
    {};
    ~FileEntry() = default;
public:
    static std::vector<unsigned char> Serialize(const FileEntry& file);
public:
    bool operator ==(const FileEntry& other) const;
};

template<>
struct std::hash<ReArchive::FileEntry> {
    std::size_t operator()(const ReArchive::FileEntry & k) const {
        std::string hash_string = k.Path().string();
        hash_string.append(std::to_string(k.Size()));
        hash_string.append(std::to_string(k.Offset()));

        return std::hash<std::string>()(hash_string);
    }
};