#pragma once

#include <cstdint>
#include <unordered_map>
#include <ReArchive/types/FileEntry.h>

namespace ReArchive {
    class FileTable;
}

class ReArchive::FileTable {
protected:
    // count
    std::unordered_map<std::filesystem::path, FileEntry> entries;
public:
    void Append(const FileEntry& file_entry);
    void Remove(const FileEntry& file_entry);
    [[nodiscard]] bool Contains(const std::filesystem::path& entry ) const { return entries.contains(entry); }
    [[nodiscard]] std::unordered_map<std::filesystem::path, FileEntry>* GetEntries() { return &entries; }
    [[nodiscard]] const std::unordered_map<std::filesystem::path, FileEntry>* GetEntries() const { return &entries; }
    [[nodiscard]] int64_t Count() const { return entries.size(); }
public:
    [[nodiscard]] static std::vector<unsigned char> Serialize(const FileTable& file_table);
public:
    FileTable(const FileTable& rhs) : entries(rhs.entries) {};
    FileTable(FileTable& rhs) : entries(rhs.entries) {};
    FileTable() = default;
    ~FileTable() = default;

};