#pragma once

#include <array>
#include <vector>
#include <cstdint>
#include <Endianness.hpp>


namespace ReArchive {
    class Header;
    static constexpr std::array<unsigned char, 3> magic { 'R', 'S', 'A' };
}

class ReArchive::Header {
protected:
    // The first 3 bytes of the file would always be this.
    std::array<unsigned char, 3> magic { 'R', 'S', 'A' };

    // Whether the file is compressed.
    bool use_compression = false;

    //TODO Whether the archive is currently in a state that we have to wait to read or write.
    //bool locked = false;

    // The distance from the beginning of the file to the "file table".
    int64_t file_table_offset;
public:
    [[nodiscard]] bool Compressed() const { return use_compression; }
    [[nodiscard]] int64_t FileTableOffset() const { return file_table_offset; }
    void FileTableOffset(const int64_t& offset) { file_table_offset = offset; }
public:
    Header(bool use_compression, int64_t file_table_offset) : use_compression(use_compression), file_table_offset(file_table_offset) {}
    ~Header() = default;
public:
    static std::vector<unsigned char> Serialize(const Header& header);
    static Header DeSerialize(const unsigned char* serialized_header);

    // We can't use sizeof(Header) because it includes the RTTI :/ - Redacted.
    static int64_t Size() { return sizeof(int64_t) + sizeof(bool) + ReArchive::magic.size(); }
};