#include <ReArchive/types/FileTable.h>
#include <cstring>
#include <Endianness.hpp>

using namespace ReArchive;

void FileTable::Append(const FileEntry& file_entry) {
    entries.try_emplace(file_entry.Path(), file_entry);
}

void FileTable::Remove(const FileEntry& file_entry) {
    if (entries.empty())
        return;

    auto position = entries.find(file_entry.Path());
    if (position != entries.end())
        entries.erase(position);
}

std::vector<unsigned char> FileTable::Serialize(const FileTable& file_table) {
    auto files = file_table.GetEntries();
    int64_t count = files->size();
    auto network_count = Endianness::HostToNetworkOrder(count);
    std::vector<unsigned char> result(reinterpret_cast<unsigned char*>(&network_count), reinterpret_cast<unsigned char*>(&network_count) + sizeof(network_count));

    if (files->empty())
        return result;

    for (const auto& file : *files) {
        size_t current_size = result.size();
        auto serialization = FileEntry::Serialize(file.second);

        result.resize(current_size + serialization.size());
        memcpy(result.data() + current_size, serialization.data(), serialization.size());
    }
    return result;
}
